function FlexibleLabel(text)
{
	return Label(text)
		.className("Farrago.FGFlexibleTextView")
		.viewName("label")
		.fillWidth()
		.textAlign(0,1)
		.intrinsicSize(-1,-1) 
		.fontSize(13)
		.fontColor(Colors.fontColor) 
		.setValue(false, "axIsIgnored")
		.lineBreakMode(0);
}

let searchResultHeight = 75

function SearchResult()
{
	return HStack([   
		Padding(2), // Small pad to account for the selection view  
		TileHostingView(false, true, true, MainThemeColor(Colors.tileBackground, Colors.secondaryBackground.brightness(1.4)))  
			.keyPath("") 
			.height(searchResultHeight, 1000)
	]) 
	.makeView("FGSearchResultTileView")   
	.childViews([
		LoadingSpinner().center(),
	])
	.setAXHelp("Press return to toggle playback. To add this clip to the current set press the command + return key")
	.setValues({
		"multipleSelection" : false, 
		selectable: true,
		drawsSelected: true,
		gxDrawsFocused: true,
	})
	.keyPath("")    
	.fillWidth()
	.intrinsicSize(-1,searchResultHeight)
	.menuItems(SearchResultMenuItems());
}

function SelectionView()
{
	return View()
		.width(2)
		.vPad(4)
		.showKey("selected")
		.keyPath("")
		.backPainter(Painter().fill(Colors.tileTint).inset(0.5)).keyPath("");
}

// Main sections

markup.push({  

	"ResultsInOtherSets" : HStack([
		SelectionView(),
		Padding(4), 
		CollapsibleHeaderStack("Tile Results in Other Sets"),
		Padding(2),
	])
	.makeView("FGSectionResultView")
	.setAXTitle("Tile Results in Other Sets")
	.setAXHelp("VO Press to expand or collapse results in other sets")
	.intrinsicSize(-1,-1)
	.fillWidth()
	.vPad(0)
	,
	
	"ResultsInSet" : VStack([ 
		Padding(),
		HStack([  
			Padding(5),
			FlexibleLabel()
				.useInlineMarkup()
				.keyPath("resultTitle") 
				.fontSize(13), 
		]),
		Padding(),
	])
	.makeView("FGSectionResultView")
	.intrinsicSize(-1,-1)
	.vPad(0)
	.fillWidth(),
	
	"FreeSoundLoginResult" : HStack([
		SelectionView(),
		Padding(4), 
		VList([
			
			CollapsibleHeaderStack("Results from Freesound").makeView(), 
			  
			VStack([
				VStack([ 
					FlexibleLabel()
						.keyPath("maxSearchesText")
						.padding(10,10,10,0)
						.fontSize(13), 
					HStack([
						PushButton("Details...") 
							.setAXTitle("Authorize Freesound account")
							.setAXHelp(`Open Freesound authorization in Farrago ${gxGlobalAppContext.osDependentPreferencesName}`)
							.width(80).keyPath("authorize"),
					]).padding(5), 
				])
				.align("left")
				.makeView() 
				.fillFrame() 
				.backPainter(Painter().fill(MainThemeColor(Colors.secondaryBackground.brightness(0.92), Colors.white.withAlpha(0.2))).radius(6)), 
				Padding(5),
			]) 
			.makeView() 
			.showKey("showLoginMessage"), 

			VStack([
				Padding(),
				FlexibleLabel() 
					.keyPath("searchError")
					.fontSize(13),  
			])
			.showKey("showSearchError"), 
			
		]) 
		.setValue(true, "gxAnimationsDisabled"),
		Padding(2),
	])  
	.makeView("FGSectionResultView")
	.setAXTitle("Results from Freesound")
	.setAXHelp("VO Press to expand or collapse Freesound results")
	.vPad(0)
	.intrinsicSize(-1,-1)  
	.fillWidth(),    
	
	"AppleLoopsHeader" : HStack([
		SelectionView(),
		Padding(4),  
		CollapsibleHeaderStack("Apple Loops"),
		Padding(2),
	]).makeView("FGSectionResultView")
	.setAXTitle("Apple Loops")
	.setAXHelp("VO Press to expand or collapse Apple Loop results")
	.intrinsicSize(-1,-1)
	.vPad(0)
	.fillWidth(),  
	
});

// Seach Results
markup.push({

	"SearchResult" : SearchResult().setAXTitle("<gx>title</gx>, Audio Tile"), 
	
	"ResultMessage" : HStack([ 
		Padding(5),
		FlexibleLabel()
			.useInlineMarkup()   
			.keyPath("message"), 
		Padding(2),
	])
	.makeView("GXSettingsView")
	.intrinsicSize(-1,-1)
	.vPad(0)
	.fillWidth(),

	"NextPageOfResultsView" : HStack([  
		View("FGPageNavigationView")
			.viewName("navigationView")
			.height(18), 
	]) 
	.makeView("GXSettingsView")
	.fillHeight()
	.intrinsicSize(-1,20)
	.setAXTitle("Page navigation")
	.fillWidth(),
	
	"FreeSoundResult" : SearchResult()
		.setAXTitle("<gx>title</gx>, Freesound Clip"),
	
	"AppleLoopResult" : SearchResult()
		.setAXTitle("<gx>title</gx>, Apple Loop"),

});

function SearchCollapseView(title)
{
	return HStack([
		Padding(),
		LoadingSpinner(15),
		//.hideKey("collapsed"),
		Padding(),
		AnimationView("disclosure", Colors.tileTint)
				.size(20)
				.keyPath("collapsed"),  
	
	]);
}

function SearchResultMenuItems()
{
	return [
		MenuItem("").setValue("playTileTitle", "gxTitleKeyPath").action("togglePlayback:"), 
		MenuItem("").setValue("previewTileTitle", "gxTitleKeyPath").action("togglePreview:"), 
	]
}

function CollapsibleHeaderStack(title)
{
	return VStack([
		Padding(5),
		HStack([ 
			FlexibleLabel(title)  
				.viewName("title")  
				.fontWeight(0.3)
				.fontSize(13),
			SearchCollapseView(),
		]), 
		Padding(4),  
	])
	.wrapInFlexibleContainer();
}

