
markup.push({ 
	"SliderPopup" : Popover(VStack([
		HStack([
			Label("Volume A:").fontSize(13).fontColor(Colors.tertiary).fontWeight(-0.4).viewName("label"),
			Padding(5),
			Label("33%").keyPath("").fontSize(13).fontColor(Colors.tileTint).fontWeight(0.3).viewName("readout"),
		]).padding(4).center()
	])
	.makeView("FGSliderPopupView") 
	.autoLoadConstraints()
	.fillWidth()
	.fillHeight()) 
	.backColor(MainThemeColor(Colors.mainBackground.lightMode, Colors.secondaryBackground.darkMode)),
});


function VolumeSlider(size=16)
{
	return Slider()  
			.height(size) 
			.knobPainter(RoundKnob().inset(1)
			.styleByMutating("disabled", (painter) => {
				return painter.fill(Colors.secondaryContent).inset(2).stroke(1, Colors.mainBackground)
			})
			.styleByMutating("muted", (painter) => {
				return painter.fill(Colors.secondaryContent).stroke(1, Colors.mainBackground)
			})
			)  
			.stylesKeyPath("volumeIndicatorStyles") 
}

function PlayerVolumeView(type) 
{
	let size = 16
	function VolumeSliderWithLetter(letter = "A")
	{ 
		return VolumeSlider(size)
			.leftPad(20)
			.rightPad(0) 
			.trackPainter(null)
			.keyPath(`volume${letter}`) 
			.viewName(`volumeSlider${letter}`)
	}
	
	function ToggleFlag(letter = "A")
	{
		return View("Farrago.FGToggleFlag")
					.ignoreAX()
					.width(14) 
					.height(15) 
					.toolTipKeyPath(`volume${letter}ToggleReadout`)
					.flipped(true)
					.setValue(Painter().fill(Colors.secondaryContent.addStyle("on", Colors.tileTint).showPressed().addStyle("locked", Colors.tertiaryContent)), "flagPainter") 
					.backPainter(
						LabelPainter(letter)
							.color(Colors.white.addStyle("8", Colors.primaryContent.inverted()))
							.offset(0, -1)
							.fontSize(13) 
							.fontFamily("SF Pro Rounded")
					)
					.keyPath(`volume${letter}Btn`)
					.viewName(`volumeSlider${letter}Flag`)
					.setAXTitle(`Volume ${letter}`)
					.setAXHelp(`Switch output volume ${letter}`);
	}
	
	return View("Farrago.FGVolumeView").childViews([
		
		VolumeIndicator()
			.toolTip("Mute output", "Unmute output")  
			.keyPath("volumeIndicator") 
			.checkBoxRole()
			.setAXTitle(`${type} Mute Button`)
			.leftPad(0)
			.topPad(15)
			.viewName("muteButton"),

		ToggleFlag("A"), 
		ToggleFlag("B"), 

		Slider()
			.topPad(0)
			.height(14)
			.leftPad(20)
			.rightPad(0)
			.trackPainter(null)
			.knobPainter(Painter().size(20).fill(Colors.invisible))
			.keyPath(`fullVolume`)  
			.viewName("flagSliderA")
			// .setValue(true, "gxIgnoreTrackClicks")
			.ignoreAX(),

		Slider()
			.topPad(0)
			.height(14)
			.leftPad(20)
			.rightPad(0)
			.trackPainter(null)
			.knobPainter(Painter().size(20).fill(Colors.invisible))
			.viewName("flagSliderB")
			.keyPath(`duckedVolume`) 
			// .setValue(true, "gxIgnoreTrackClicks") 
			.ignoreAX(),
		
		Slider()
			.height(size)
			.topPad(15)
			.leftPad(20)
			.rightPad(0)
			.knobPainter(null)
			.keyPath("") 
			.disableKey(`muted`)
			.viewName("volumeSliderBack"),  
		
		VolumeSliderWithLetter("A")  
			.topPad(15)
			.disableKey("ducked"),
		VolumeSliderWithLetter("B")  
			.topPad(15)
			.enableKey("ducked"),

		Slider()
			.height(size)
			.className("Farrago.FGVolumeSliderAX")
			.leftPad(0)
			.rightPad(0)
			.backPainter(null)
			.knobPainter(null)
			.forePainter(null)
			.trackPainter(null)
			.topPad(15)
			.keyPath("currentVolume") 
			.setAXFormatter(ScaledFormatter("% .0f%%", 100, 0, 100)) 
			.viewName("volumeSliderAX")
			.setAXTitle(`${type} Volume`)
			.setAXHelp(`VO Press to toggle between volume A and B`),
	])   
	.viewName("volumeSection")
	.height(46); 
}  

function VolumeIndicator(tintColor = Colors.secondaryContent)
{
	return Button().setValues({
		gxIconBaseName: "volume-ui", 
		gxLayerZPosition: 20,
		gxAnimateIconStyles: true, 
		gxForePainter: {
			nsClass: "GXImagePainter",
			gxTintColor: tintColor,
			"muted": {
				gxTintColor: "name:muted"
			},
			"pressed": {
				gxAlpha: 0.8
			}
		},
	})
	.backPainter(null)
	.size(16)
	.viewName("volumeIndicator")
	.stylesKeyPath("volumeIndicatorStyles")
}